#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Author: lvbiao
# Created on 2019/1/22
from factdatasource.persistence.jdbc.dialect.MysqlJdbcFactDatasourceContext import MysqlJdbcFactDatasourceContext


class VernoxJdbcFactDatasourceContext(MysqlJdbcFactDatasourceContext):
    pass