class ServiceNoInstanceException(Exception):
    pass

class AuthException(Exception):
    pass

class ServiceInnerException(Exception):
    pass

class IllegalArgumentException(Exception):
    pass

class UnSupportException(Exception):
    pass